<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 406c9a8b510c $
 * $Author: Lukas Imhof $
 * $Date: 2020-10-04 22:24 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * General Definition of WebEdition Glossary
 *
 */
class we_glossary_glossary extends we_base_model{
	const TYPE_LINK = 'link';
	const TYPE_ACRONYM = 'acronym';
	const TYPE_ABBREVATION = 'abbreviation';
	const TYPE_FOREIGNWORD = 'foreignword';
	const TYPE_TEXTREPLACE = 'textreplacement';
	const MODE_CATEGORY_INT = 'intern';
	const MODE_CATEGORY_EXT = 'extern';
	const MODE_LINK_INT = 'intern';
	const MODE_LINK_EXT = 'extern';
	const MODE_LINK_OBJ = 'object';
	const MODE_LINK_CAT = 'category';

	/** @var int Identifier of the glossayry item	 */
	var $ID = 0;

	/** @var string Path of the glossary item, composed by the language type and text	 */
	var $Path = '';

	/** @var bool is the item a folder */
	var $IsFolder = false;

	/** @var string Text of the item	 */
	var $Text = '';

	/** @var string Type of the item, could be abbreviation, acronym, foreignword or link or textreplacement	 */
	var $Type = '';

	/** @var string Language of the item	 */
	var $Language = '';

	/** @var string title attribute	 */
	var $Title = '';

	/** @var array all valid other attributes needed for later replacement	 */
	var $Attributes = [];

	/** @var bool should the item be linked to a detailed description	 */
	var $Linked = false;

	/** @var string detailed description of the item	 */
	var $Description = '';

	/** @var DateTime|null timestamp of publishing */
	var $Publish_Date = null;

	/**  @var int id of creator	 */
	var $CreatorID = 0;

	/** @var int id of modifier	 */
	var $ModifierID = 0;

	/** @var bool determines if the whole word is replaced  */
	public $Fullword = true;

	/**
	 * @param integer $GlossaryId
	 * @desc Could load a glossary item if $GlossaryId is not 0
	 */
	public function __construct(int $GlossaryId = 0){
		$this->persistent_slots = [
			'ID' => we_base_request::INT,
			'Path' => we_base_request::STRING,
			'IsFolder' => we_base_request::BOOL,
			'Text' => we_base_request::STRING,
			'Type' => we_base_request::STRING,
			'Language' => we_base_request::STRING,
			'Title' => we_base_request::STRING,
			'Attributes' => we_base_request::SERIALIZED_JSON,
			'Linked' => we_base_request::BOOL,
			'Description' => we_base_request::STRING,
			'Fullword' => we_base_request::BOOL,
			'Publish_Date' => we_base_request::DATE_TIME,
			'CreatorID' => we_base_request::INT,
			'ModifierID' => we_base_request::INT,
		];
		parent::__construct(GLOSSARY_TABLE);

		if($GlossaryId){
			$this->ID = $GlossaryId;
			$this->load($this->ID);
			return;
		}
		switch(we_base_request::string('cmd')){
			case 'new_glossary_abbreviation':
				$this->Type = self::TYPE_ABBREVATION;
				break;
			case 'new_glossary_acronym':
				$this->Type = self::TYPE_ACRONYM;
				break;
			case 'new_glossary_foreignword':
				$this->Type = self::TYPE_FOREIGNWORD;
				break;
			case 'new_glossary_link':
				$this->Type = self::TYPE_LINK;
				break;
			case 'new_glossary_textreplacement':
				$this->Type = self::TYPE_TEXTREPLACE;
				break;
		}
		$cmdid = we_base_request::string('cmdid');
		if($cmdid && !is_numeric($cmdid)){
			$this->Language = substr($cmdid, 0, 5);
		}
	}

	/**
	 *
	 * @param string $Language
	 * @param string $Mode
	 * @param string $Type
	 * @return array{Type:string,Text:string,Title:string,Lang:string}[]

	 */
	public static function getEntries(string $Language, string $Mode = 'all', string $Type = 'all'): array{
		$Query = 'SELECT
Type,
Text,
Title,
Attributes
FROM ' .
			GLOSSARY_TABLE . '
WHERE
	Language="' . DB_WE::$inst->escape($Language) . '" ' .
			($Type != 'all' ? 'AND
	Type="' . DB_WE::$inst->escape($Type) . '" ' : '');

		switch($Mode){
			case 'published':
				$Query .= 'AND IsPublished=1 ';
				break;
			case 'unpublished':
				$Query .= 'AND IsPublished=0 ';
				break;
		}

		$all = DB_WE::$inst->getAllq($Query);
		'@phan-var array{Type:string,Text:string,Title:string,Lang:string,Attributes:string}[] $all';

		foreach($all as &$Item){
			if($Item['Type'] != self::TYPE_FOREIGNWORD){
				$temp = we_unserialize($Item['Attributes']);
				$Item['Lang'] = ($temp['lang'] ?? '');
			} else {
				$Item['Lang'] = '';
			}
			unset($Item['Type']);
		}
		return $all;
	}

	function publishItem(string $Language, string $Text): bool{
		return DB_WE::$inst->query('UPDATE ' . GLOSSARY_TABLE . ' SET Publish_Date=NOW()'
				. ' WHERE Language="' . DB_WE::$inst->escape($Language) . '" AND Text="' . DB_WE::$inst->escape($Text) . '"');
	}

	/**
	 * save the item to the database
	 *
	 */
	function save(bool $force_new = false, bool $rebuild = false, array $skipWrite = []): bool{
		$this->setPath();

		if(!$this->ID){
			$this->CreatorID = we_users_user::getID();
		}
		$this->ModifierID = we_users_user::getID();

		$retVal = parent::save();

		/* if(!$this->ID){
		  $this->ID = $this->db->getInsertId();
		  } */

		if($retVal){
			$this->registerMediaLinks();
		}

		return $retVal;
	}

	function registerMediaLinks(): bool{
		$this->unregisterMediaLinks();
		if($this->Type === self::TYPE_LINK && !intval($this->IsFolder)){
			if(!empty($this->Attributes) && $this->Attributes['mode'] === self::MODE_LINK_INT && $this->Attributes['InternLinkID']){
				$this->MediaLinks[] = $this->Attributes['InternLinkID'];
			}
		}

		return parent::registerMediaLinks();
	}

	/**
	 * delete a glossary item from database
	 *
	 * @return boolean
	 */
	function delete(): bool{
		if((!$this->ID) || ($this->IsFolder && !$this->deleteChilds())){
			return false;
		}

		return parent::delete();
	}

	/**
	 * delete all childs of a item
	 *
	 * @return boolean
	 */
	protected function deleteChilds(): bool{
		return $this->db->query('DELETE FROM ' . $this->db->escape($this->table) . ' WHERE Path LIKE "' . $this->db->escape($this->Path) . '/%"');
	}

	/**
	 * set the path of the item
	 *
	 */
	function setPath(): void{
		$this->Path = '/' . $this->Language . '/' . $this->Type . '/' . $this->Text;
	}

	/**
	 * set an Attribute
	 *
	 * @param string $Name
	 * @param string $Value
	 */
	function setAttribute(string $Name, string $Value): void{
		$this->Attributes[$Name] = $Value;
	}

	/**
	 * get a attribute
	 *
	 * @param string $Name
	 * @return mixed
	 */
	function getAttribute(string $Name): string{
		if(!array_key_exists($Name, $this->Attributes)){
			return '';
		}

		return $this->Attributes[$Name];
	}

	/**
	 * checks if a path already exists
	 *
	 * @param string $Path
	 * @return boolean
	 */
	function pathExists(string $Path): bool{
		return fB('SELECT 1 FROM ' . $this->db->escape($this->table) . " WHERE Path LIKE Binary '" . $this->db->escape($Path) . "'" . ($this->ID ? ' AND ID != ' . intval($this->ID) : '') . ' LIMIT 1');
	}

	function getIDByPath(string $Path): int{
		return fI('SELECT ID FROM ' . $this->db->escape($this->table) . ' WHERE Path="' . $this->db->escape($Path) . '"');
	}

	/**
	 * check if the item is self (?!)
	 *
	 * @return boolean
	 */
	public function isSelf(): bool{
		return strpos(htmlentities(we_base_file::clearPath(dirname($this->Path)) . '/'), '/' . self::escapeChars($this->Text) . '/') !== false;
	}

	private static function escapeChars(string $Text): string{
		$Text = quotemeta($Text); // escape . \ + * ? [ ^ ] ( $ )

		$escape = ['{', '&', '/', '\'', '"', '%'];

		foreach($escape as $k){
			$before = $k;
			$after = "\\" . $k;
			if($k != ''){
				$Text = str_replace($before, $after, $Text);
			}
		}
		return $Text;
	}

	/**
	 * Clear all data from session
	 *
	 */
	function clearSessionVars(): void{
		if(isset($_SESSION['weS']['module_sessions']['glossary'])){
			unset($_SESSION['weS']['module_sessions']['glossary']);
		}
	}

	public static function addToException(string $language, string $entry = ''): bool{
		if(trim($entry) === ''){
			return true;
		}

		$items = self::getException($language);
		$items[] = $entry;

		return self::editException($language, implode("\n", $items));
	}

	public static function editException(string $language, string $entries): bool{
		$fileName = self::getExceptionFilename($language);
		$items = explode("\n", $entries);
		sort($items);

		return we_base_file::save($fileName, implode("\n", array_filter(array_map('trim', $items))));
	}

	public static function getException(string $language): array{
		$fileName = self::getExceptionFilename($language);

		if(file_exists($fileName) && is_file($fileName)){
			return file($fileName) ?: [];
		}

		return [];
	}

	public static function getExceptionFilename(string $language): string{
		$fileDir = WE_INCLUDES_PATH . 'glossary_dict/';
		return $fileDir . $language . '@' . $_SERVER['SERVER_NAME'] . '.dict';
	}

	function checkFieldText(string $text): bool{
		$check = ['\\', '$', '|'];

		foreach($check as $k){
			if(stristr(trim($text), $k)){
				return true;
			}
		}
		return false;
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
				'checking' => g_l('modules_glossary', '[checking]'),
				'all_words_identified' => g_l('modules_glossary', '[all_words_identified]'),
				'change_to' => g_l('modules_glossary', '[change_to]'),
				'input' => g_l('modules_glossary', '[input]'),
				'suggestions' => g_l('modules_glossary', '[suggestions]'),
				'languages' => g_l('modules_glossary', '[languages]'),
				'please_insert_title' => (g_l('modules_glossary', '[please_insert_title]')),
				'please_insert_language' => (g_l('modules_glossary', '[please_insert_language]')),
				'please_insert_correct_word' => (g_l('modules_glossary', '[please_insert_correct_word]')),
				'please_choose_action' => (g_l('modules_glossary', '[please_choose_action]')),
				'view' => [
					'delete_alert' => g_l('modules_glossary', '[delete_alert]'),
					'no_perms' => (g_l('modules_glossary', '[no_perms]')),
					'no_workspace' => (g_l('modules_glossary', '[no_workspace]')),
					'nothing_to_delete' => (g_l('modules_glossary', '[nothing_to_delete]')),
					'nothing_to_save' => (g_l('modules_glossary', '[nothing_to_save]')),
					'save_changed_glossary' => g_l('modules_glossary', '[save_changed_glossary]'),
				],
		]);
	}

	public static function getJSConsts(): string{
		return we_base_util::setLangString([
				'TYPE_ABBREVATION' => we_glossary_glossary::TYPE_ABBREVATION,
				'TYPE_ACRONYM' => we_glossary_glossary::TYPE_ACRONYM,
				'TYPE_FOREIGNWORD' => we_glossary_glossary::TYPE_FOREIGNWORD,
		]);
	}

}
