<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6cc64d37551c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-25 11:32 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Filename:    we_html_element.inc.php
 *
 * Function:    Class to create html tags
 *
 * Description: Provides functions for creating html tags
 */
abstract class we_html_element{

	/**
	 * Function generates html code for html form
	 *
	 * @param		array<string,string>	$attribs (optional)
	 * @param	string	$content	(optional)
	 *
	 * @return		string
	 */
	public static function form(array $attribs = [], string $content = ''): string{

		if(!isset($attribs['name'])){
			$attribs['name'] = we_base_constants::WE_FORM;
		}
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('form', true, $attribs, $content));
	}

	/**
	 * Function generates html code for html input element
	 *
	 * @param		array<string,string>	$attribs										(optional)
	 * @return		string
	 */
	public static function input(array $attribs = []): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('input', we_html_baseElement::SELF_CLOSE, $attribs));
	}

	/**
	 * Function generates css code
	 *
	 * @param		string $content											(optional)
	 * @param		array<string,string>	$attribs										(optional)
	 *
	 * @return		string
	 */
	public static function cssElement(string $content = '', array $attribs = []): string{
		$attribs['type'] = 'text/css';

		return we_html_baseElement::getHtmlCode(new we_html_baseElement('style', true, $attribs, $content));
	}

	/**
	 *
	 * @param string $name
	 * @param string $onload
	 * @param		array<string,string>	$attribs										(optional)
	 * @return string
	 */
	public static function jsScript(string $name, string $onload = '', array $attribs = []): string{
		$attribs['src'] = self::getUnCache($name);
		if($onload){
			$attribs['onload'] = $onload;
		}
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('script', true, $attribs));
	}

	/**
	 * Function generates js code
	 *
	 * @param		string $content											(optional)
	 * @param		array<string,string>	$attribs										(optional)
	 *
	 * @return		string
	 */
	public static function jsElement(string $content = '', array $attribs = []): string{
		if(strpos($content, '<!--') === false){
			$content = "<!--\n" . trim($content, " \n") . "\n//-->\n";
		}
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('script', true, $attribs, $content));
	}

	/**
	 *
	 * @param string $url
	 * @param		array<string,string>	$attribs										(optional)
	 * @return string
	 */
	public static function cssLink(string $url, array $attribs = []): string{
		$attribs['href'] = self::getUnCache($url);
		$attribs['rel'] = 'styleSheet';
		$attribs['type'] = 'text/css';
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('link', false, $attribs));
	}

	/**
	 * Function generates link code
	 *
	 * @param		array<string,string>	$attribs										(optional)
	 *
	 * @return		string
	 */
	public static function linkElement(array $attribs = []): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('link', we_html_baseElement::SELF_CLOSE, $attribs));
	}

	/**
	 * Function generates html code for html div elements
	 *
	 * @param		array<string,string>	$attribs										(optional)
	 * @param		string $content											(optional)
	 *
	 * @return		string
	 */
	public static function span(array $attribs = [], string $content = ''): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('span', true, $attribs, $content));
	}

	//FIMXE: remove for htmltools
	public static function select(array $attribs = [], string $content = ''): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('select', true, $attribs, $content));
	}

	/**
	 * Function generates html code for html div elements
	 *
	 * @param		array<string,string>	$attribs										(optional)
	 * @param		string $content											(optional)
	 *
	 * @return		string
	 */
	public static function div(array $attribs = [], string $content = ''): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('div', true, $attribs, $content));
	}

	/**
	 * Function generates html code for html b element
	 *
	 * @param	string	$content
	 *
	 * @return		string
	 */
	public static function b(string $content): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('strong', true, [], $content));
	}

	/**
	 * Function generates html code for html image element
	 *
	 * @param		array<string,string>	$attribs										(optional)
	 *
	 * @return		string
	 */
	public static function img(array $attribs = []): string{
		//if no alt is set, set dummy alt
		if(!isset($attribs['alt'])){
			$attribs['alt'] = '-';
		}
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('img', we_html_baseElement::SELF_CLOSE, $attribs));
	}

	/**
	 * Function generates html code for html body element
	 *
	 * @param		array<string,string>	$attribs										(optional)
	 * @param		string $content											(optional)
	 *
	 * @return		string
	 */
	public static function body(array $attribs = [], string $content = ''): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('body', true, $attribs, $content));
	}

	/**
	 * Function generates html code for html label element
	 *
	 * @param		array<string,string>	$attribs										(optional)
	 * @param		string $content											(optional)
	 *
	 * @return		string
	 */
	public static function label(array $attribs = [], string $content = ''): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('label', true, $attribs, $content));
	}

	/**
	 * Function generates html code for html hidden element
	 *
	 * @param		array<string,string>	$attribs										(optional)
	 *
	 * @return		string
	 */
	public static function hidden(string $name, /* mixed */ $value, string $id = '', array $attribs = []): string{
		$attribs = array_merge($attribs, [
			'type' => 'hidden',
			'name' => $name,
			'value' => is_string($value) && strpos($value, '"') !== false ? htmlspecialchars($value) : $value
		]);
		if($id){
			$attribs['id'] = $id;
		}
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('input', we_html_baseElement::SELF_CLOSE, $attribs));
	}

	/**
	 *
	 * @param		array<string,mixed>	$vals
	 * @return string
	 */
	public static function hiddens(array $vals): string{
		$ret = '';
		foreach($vals as $key => $value){
			if($key){
				$ret .= we_html_baseElement::getHtmlCode(new we_html_baseElement('input', we_html_baseElement::SELF_CLOSE, [
							'name' => $key,
							'value' => is_string($value) && strpos($value, '"') !== false ? htmlspecialchars($value) : (string) $value,
							'type' => 'hidden'
				]));
			}
		}
		return $ret;
	}

	/**
	 * Function generates html code for html a element
	 * @param int $level
	 * @param		string $content
	 * @param		array<string,string> $attribs								(optional)
	 *
	 * @return		string
	 */
	public static function h(int $level, string $content = '', array $attribs = []): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('h' . $level, true, $attribs, $content));
	}

	/**
	 * Function generates html code for html a element
	 *
	 * @param		array<string,string>	$attribs										(optional)
	 * @param		string			$content								(optional)
	 *
	 * @return		string
	 */
	public static function a(array $attribs = [], string $content = ''): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('a', true, $attribs, $content));
	}

	/**
	 * Function generates html code for html br element
	 * @param int $count no of breaks
	 * @return		string
	 */
	public static function br(int $count = 1): string{
		return str_repeat('<br/>', $count);
	}

	/**
	 * Function generates html code for html br element
	 *
	 * @param		string $content											(optional)
	 *
	 * @return		string
	 */
	public static function comment(string $content): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('!-- ' . $content . ' --', false));
	}

	/**
	 *
	 */
	public static function docType(/* string $version = '5' */): string{
		/* 	switch($version){
		  default:
		  case '5': */
		return '<!DOCTYPE html>';
		/* case '4Trans':
		  return '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">';
		  } */
	}

	/**
	 * Function generates html code for html document
	 *
	 * @return		string
	 */
	public static function html(string $content, bool $close = true): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('html', $close, [/* 'lang' => '', 'xml:lang' => '' */], $content));
	}

	/**
	 * Function generates html code for document head
	 *
	 * @return		string
	 */
	public static function head(string $content, bool $close = true): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('head', $close, [], $content));
	}

	public static function meta(array $attribs = []): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('meta', we_html_baseElement::SELF_CLOSE, $attribs));
	}

	public static function title(string $content): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('title', true, [], $content));
	}

	/**
	 * Function generates html code for textarea tag
	 * @param		array<string,string>	$attribs										(optional)
	 * @return		string
	 */
	public static function textArea(array $attribs = [], string $content = ''): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('textarea', true, $attribs, $content));
	}

	/**
	 * Function generates html code for p tag
	 * @param		array<string,string>	$attribs										(optional)
	 * @return		string
	 */
	public static function ul(array $attribs = [], string $content = ''): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('ul', true, $attribs, $content));
	}

	/**
	 * Function generates html code for p tag
	 * @param		array<string,string>	$attribs										(optional)
	 * @return		string
	 */
	public static function li(array $attribs = [], string $content = ''): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('li', true, $attribs, $content));
	}

	/**
	 * Function generates html code for p tag
	 * @param		array<string,string>	$attribs										(optional)
	 * @return		string
	 */
	public static function p(array $attribs = [], string $content = ''): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('p', true, $attribs, $content));
	}

	/**
	 * Function generates html code for center tag
	 *
	 * @return		string
	 */
	public static function center(string $content): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('center', true, [], $content));
	}

	/**
	 * Function generates html code for center tag
	 *
	 * @return		string
	 */
	public static function applet(array $attribs = [], string $content = '', array $params = []): string{
		//$params['cache_archive'] = $attribs['archive']; // Applet seams not to like this param
		$params['cache_version'] = WE_VERSION;
		$params['type'] = 'application/x-java-applet;jpi-version=1.6.0';
		$params['scriptable'] = 'true';
		$params['mayscript'] = 'true';
		$tmp = '';
		foreach($params as $key => $value){
			$tmp .= self::param(['name' => $key, 'value' => $value]);
		}
		$content = $tmp . $content;
		$attribs['MAYSCRIPT'] = '';
		$attribs['SCRIPTABLE'] = '';


		return we_html_baseElement::getHtmlCode(new we_html_baseElement('applet', true, $attribs, $content));
	}

	/**
	 * Function generates html code for center tag
	 * @param		array<string,string>	$attribs										(optional)
	 * @return		string
	 */
	public static function param(array $attribs = []): string{
		return we_html_baseElement::getHtmlCode(new we_html_baseElement('param', we_html_baseElement::SELF_CLOSE, $attribs));
	}

	/**
	 * this func is used to get a parameter variing in each version, to get the latest file (browser cache)
	 * but don't offer information about the current installed version!
	 * @staticvar string $cache saves current cached path
	 * @param string $url url to add the version-unique param
	 * @return string resulting url
	 */
	public static function getUnCache(string $url): string{
		static $cache = -1;
		if($cache == -1){
			$cache = md5(WE_VERSION . filemtime(WE_INCLUDES_PATH . 'we_version.php'));
		}
		return $url . (strpos($url, '?') !== false ? '&amp;' : '?') . $cache;
	}

	public static function iframe(string $name, string $src, string $style = '', string $iframestyle = '', string $onload = '', string $class = ''): string{
		static $isAppleTouch = -1;
		$isAppleTouch = ($isAppleTouch !== -1 ? $isAppleTouch : (/* we_base_browserDetect::inst()->isSafari() */ (we_base_browserDetect::inst()->getSystem() == we_base_browserDetect::SYS_IPAD || we_base_browserDetect::inst()->getSystem() == we_base_browserDetect::SYS_IPHONE)));
		$iframestyle = $iframestyle ?: 'border:0;width:100%;height:100%;overflow:hidden;';

		return self::div(['style' => $style, 'name' => $name . 'Div', 'id' => $name . 'Div', 'class' => $class . ($isAppleTouch ? ' iframeScrollIpad' : '')]
				, we_html_baseElement::getHtmlCode(new we_html_baseElement('iframe', true, ['name' => $name, 'id' => $name, 'src' => $src, 'style' => $iframestyle, 'class' => ($isAppleTouch ? 'iframeScrollIpad' : ''),
						'onload' => $onload])
		));
	}

	public static function exIFrame(string $divname, string $src, string $style = '', string $class = ''): string{
		if(strpos($src, WE_DOCUMENT_ROOT) === 0){
			ob_start();
			include $src;
			$tmp = (string) (ob_get_clean() ?? '');
		} else {
			$tmp = $src;
		}
		return self::div(['style' => $style, 'name' => $divname . 'Div', 'id' => $divname . 'Div', 'class' => $class], $tmp);
	}

}
